@@ -1,21 +1,22 @@
 #!/usr/bin/env python
 """Nile CLI entry point."""
+
 import logging
 import os
+from functools import update_wrapper
 
 import asyncclick as click
 
 from nile.common import is_alias
-from nile.core.account import Account, get_counterfactual_address
 from nile.core.call_or_invoke import call_or_invoke as call_or_invoke_command
 from nile.core.clean import clean as clean_command
 from nile.core.compile import compile as compile_command
-from nile.core.deploy import deploy as deploy_command
 from nile.core.init import init as init_command
 from nile.core.node import node as node_command
 from nile.core.plugins import load_plugins
 from nile.core.run import run as run_command
 from nile.core.test import test as test_command
+from nile.core.types.account import get_counterfactual_address, try_get_account
 from nile.core.version import version as version_command
 from nile.signer import Signer
 from nile.utils import hex_address, normalize_number, shorten_address
@@ -33,6 +34,26 @@ logging.getLogger("asyncio").setLevel(logging.WARNING)
 NETWORKS = ("localhost", "integration", "goerli", "goerli2", "mainnet")
 
 
+def enable_stack_trace(f):
+    """Enable stack trace swapping for commands."""
+
+    @click.pass_context
+    async def new_func(ctx, *args, **kwargs):
+        if ctx.obj["STACK_TRACE"]:
+            return await ctx.invoke(f, ctx.obj, *args, **kwargs)
+        else:
+            try:
+                return await ctx.invoke(f, ctx.obj, *args, **kwargs)
+            except Exception as e:
+                logging.error(
+                    "The following exception occured while "
+                    f"trying to execute the command:\n\n{repr(e)}\n\n"
+                    "Try `nile --stack_trace [COMMAND]` for the full stack trace."
+                )
+
+    return update_wrapper(new_func, f)
+
+
 def network_option(f):
     """Configure NETWORK option for the cli."""
     return click.option(  # noqa: E731
@@ -51,12 +72,21 @@ def watch_option(f):
     return f
 
 
-def mainnet_token_option(f):
-    """Configure TOKEN option for the cli."""
-    return click.option(
-        "--token",
-        help="Used for deploying contracts in Alpha Mainnet.",
-    )(f)
+def query_option(f):
+    """Handle simulate and estimate_fee options for the cli."""
+    f = click.option("--simulate", "query", flag_value="simulate")(f)
+    f = click.option("--estimate_fee", "query", flag_value="estimate_fee")(f)
+    return f
+
+
+async def run_transaction(tx, query_flag, watch_mode):
+    """Execute or simulate a transaction."""
+    if query_flag == "estimate_fee":
+        await tx.estimate_fee()
+    elif query_flag == "simulate":
+        await tx.simulate()
+    else:
+        await tx.execute(watch_mode=watch_mode)
 
 
 def _validate_network(_ctx, _param, value):
@@ -75,13 +105,19 @@ def _validate_network(_ctx, _param, value):
 
 
 @click.group()
-def cli():
+@click.option("--stack_trace/--no_stack_trace", default=False)
+@click.pass_context
+def cli(ctx, stack_trace):
     """Nile CLI group."""
-    pass
+    # ensure that ctx.obj exists and is a dict
+    ctx.ensure_object(dict)
+
+    ctx.obj["STACK_TRACE"] = stack_trace
 
 
 @cli.command()
-def init():
+@enable_stack_trace
+def init(ctx):
     """Nile CLI group."""
     init_command()
 
@@ -89,7 +125,8 @@ def init():
 @cli.command()
 @click.argument("path", nargs=1)
 @network_option
-async def run(path, network):
+@enable_stack_trace
+async def run(ctx, path, network):
     """Run Nile scripts with NileRuntimeEnvironment."""
     await run_command(path, network)
 
@@ -98,21 +135,18 @@ async def run(path, network):
 @click.argument("signer", nargs=1)
 @click.argument("contract_name", nargs=1)
 @click.argument("params", nargs=-1)
-@click.option("--max_fee", nargs=1)
-@click.option("--salt", nargs=1, default=0)
+@click.option("--max_fee", type=int, nargs=1)
+@click.option("--salt", type=int, nargs=1, default=0)
 @click.option("--unique", is_flag=True)
 @click.option("--alias")
 @click.option("--abi")
 @click.option("--deployer_address")
-@click.option(
-    "--ignore_account",
-    is_flag=True,
-    help="Deploy without Account.",
-)
-@mainnet_token_option
 @network_option
+@query_option
 @watch_option
+@enable_stack_trace
 async def deploy(
+    ctx,
     signer,
     contract_name,
     salt,
@@ -122,86 +156,86 @@ async def deploy(
     alias,
     abi,
     deployer_address,
-    ignore_account,
-    token,
     network,
+    query,
     watch_mode,
 ):
     """Deploy a StarkNet smart contract."""
-    if not ignore_account:
-        account = await Account(signer, network, watch_mode=watch_mode)
-        await account.deploy_contract(
+    account = await try_get_account(signer, network, watch_mode="track")
+    if account is not None:
+        transaction = await account.deploy_contract(
             contract_name,
             salt,
             unique,
             params,
-            alias,
             deployer_address=deployer_address,
+            alias=alias,
             max_fee=max_fee,
             abi=abi,
-            watch_mode=watch_mode,
-        )
-    else:
-        await deploy_command(
-            contract_name,
-            params,
-            network,
-            alias,
-            abi=abi,
-            mainnet_token=token,
-            watch_mode=watch_mode,
         )
 
+        await run_transaction(tx=transaction, query_flag=query, watch_mode=watch_mode)
+
 
 @cli.command()
 @click.argument("signer", nargs=1)
 @click.argument("contract_name", nargs=1)
-@click.option("--max_fee", nargs=1)
+@click.option("--max_fee", type=int, nargs=1)
 @click.option("--alias")
 @click.option("--overriding_path")
 @click.option("--nile_account", is_flag=True)
 @network_option
-@mainnet_token_option
+@query_option
 @watch_option
+@enable_stack_trace
 async def declare(
+    ctx,
     signer,
     contract_name,
     network,
     max_fee,
-    watch_mode,
     alias,
     overriding_path,
-    token,
+    query,
     nile_account,
+    watch_mode,
 ):
     """Declare a StarkNet smart contract through an Account."""
-    account = await Account(signer, network, watch_mode=watch_mode)
-    await account.declare(
-        contract_name,
-        alias=alias,
-        max_fee=max_fee,
-        overriding_path=overriding_path,
-        mainnet_token=token,
-        watch_mode=watch_mode,
-        nile_account=nile_account,
-    )
+    account = await try_get_account(signer, network, watch_mode="track")
+    if account is not None:
+        transaction = await account.declare(
+            contract_name,
+            max_fee=max_fee,
+            alias=alias,
+            overriding_path=overriding_path,
+            nile_account=nile_account,
+        )
+
+        await run_transaction(tx=transaction, query_flag=query, watch_mode=watch_mode)
 
 
 @cli.command()
 @click.argument("signer", nargs=1)
-@click.option("--salt", nargs=1)
-@click.option("--max_fee", nargs=1)
+@click.option("--salt", type=int, nargs=1)
+@click.option("--max_fee", type=int, nargs=1)
 @network_option
+@query_option
 @watch_option
-async def setup(signer, network, salt, max_fee, watch_mode):
+@enable_stack_trace
+async def setup(ctx, signer, network, salt, max_fee, query, watch_mode):
     """Set up an Account contract."""
-    await Account(signer, network, salt, max_fee, watch_mode=watch_mode)
+    account = await try_get_account(signer, network, auto_deploy=False)
+    if account is not None:
+        transaction = await account.deploy(salt, max_fee)
+
+        await run_transaction(tx=transaction, query_flag=query, watch_mode=watch_mode)
 
 
 @cli.command()
 @click.argument("signer", nargs=1)
-@click.option("--salt", nargs=1, default=None)
-def counterfactual_address(signer, salt):
+@click.option("--salt", type=int, nargs=1)
+@enable_stack_trace
+def counterfactual_address(ctx, signer, salt):
     """Precompute the address of an Account contract."""
     _signer = Signer(normalize_number(os.environ[signer]))
     address = hex_address(
@@ -215,12 +249,13 @@ def counterfactual_address(signer, salt):
 @click.argument("address_or_alias", nargs=1)
 @click.argument("method", nargs=1)
 @click.argument("params", nargs=-1)
-@click.option("--max_fee", nargs=1)
-@click.option("--simulate", "query", flag_value="simulate")
-@click.option("--estimate_fee", "query", flag_value="estimate_fee")
+@click.option("--max_fee", type=int, nargs=1)
 @network_option
+@query_option
 @watch_option
+@enable_stack_trace
 async def send(
+    ctx,
     signer,
     address_or_alias,
     method,
@@ -231,22 +266,22 @@ async def send(
     watch_mode,
 ):
     """Invoke a contract's method through an Account."""
-    account = await Account(signer, network, watch_mode=watch_mode)
-    print(
-        "Calling {} on {} with params: {}".format(
-            method, address_or_alias, [x for x in params]
+    account = await try_get_account(signer, network, watch_mode="track")
+    if account is not None:
+        print(
+            "Calling {} on {} with params: {}".format(
+                method, address_or_alias, [x for x in params]
+            )
         )
-    )
-    # address_or_alias is not normalized first here because
-    # Account.send is part of Nile's public API and can accept hex addresses
-    await account.send(
-        address_or_alias,
-        method,
-        params,
-        max_fee=max_fee,
-        query_type=query,
-        watch_mode=watch_mode,
-    )
+
+        transaction = await account.send(
+            address_or_alias,
+            method,
+            params,
+            max_fee=max_fee,
+        )
+
+        await run_transaction(tx=transaction, query_flag=query, watch_mode=watch_mode)
 
 
 @cli.command()
@@ -254,7 +289,8 @@ async def send(
 @click.argument("method", nargs=1)
 @click.argument("params", nargs=-1)
 @network_option
-async def call(address_or_alias, method, params, network):
+@enable_stack_trace
+async def call(ctx, address_or_alias, method, params, network):
     """Call functions of StarkNet smart contracts."""
     if not is_alias(address_or_alias):
         address_or_alias = normalize_number(address_or_alias)
@@ -288,8 +324,9 @@ def test(contracts):
 @click.option("--cairo_path")
 @click.option("--account_contract", is_flag="True")
 @click.option("--disable-hint-validation", is_flag=True)
+@enable_stack_trace
 def compile(
-    contracts, directory, cairo_path, account_contract, disable_hint_validation
+    ctx, contracts, directory, cairo_path, account_contract, disable_hint_validation
 ):
     """
     Compile cairo contracts.
@@ -309,7 +346,8 @@ def compile(
 
 
 @cli.command()
-def clean():
+@enable_stack_trace
+def clean(ctx):
     """Remove default build directory."""
     clean_command()
 
@@ -319,7 +357,8 @@ def clean():
 @click.option("--port", default=5050)
 @click.option("--seed", type=int)
 @click.option("--lite_mode", is_flag=True)
-def node(host, port, seed, lite_mode):
+@enable_stack_trace
+def node(ctx, host, port, seed, lite_mode):
     """Start StarkNet local network.
 
     $ nile node
@@ -339,7 +378,8 @@ def node(host, port, seed, lite_mode):
 
 @cli.command()
 @click.version_option()
-def version():
+@enable_stack_trace
+def version(ctx):
     """Print out toolchain version."""
     version_command()
 
@@ -348,7 +388,8 @@ def version():
 @click.argument("tx_hash", nargs=1)
 @click.option("--contracts_file", nargs=1)
 @network_option
-async def debug(tx_hash, network, contracts_file):
+@enable_stack_trace
+async def debug(ctx, tx_hash, network, contracts_file):
     """
     Locate an error in a transaction using available contracts.
 
@@ -362,7 +403,8 @@ async def debug(tx_hash, network, contracts_file):
 @click.option("--contracts_file", nargs=1)
 @network_option
 @watch_option
-async def status(tx_hash, network, watch_mode, contracts_file):
+@enable_stack_trace
+async def status(ctx, tx_hash, network, watch_mode, contracts_file):
     """
     Get the status of a transaction.
 
@@ -386,7 +428,8 @@ async def status(tx_hash, network, watch_mode, contracts_file):
 @cli.command()
 @click.option("--predeployed/--registered", default=False)
 @network_option
-async def get_accounts(network, predeployed):
+@enable_stack_trace
+async def get_accounts(ctx, network, predeployed):
     """Retrieve and manage deployed accounts."""
     if not predeployed:
         await get_accounts_command(network)
@@ -397,7 +440,8 @@ async def get_accounts(network, predeployed):
 @cli.command()
 @click.argument("contract_address")
 @network_option
-async def get_balance(contract_address, network):
+@enable_stack_trace
+async def get_balance(ctx, contract_address, network):
     """Retrieve the Ether balance for a given contract."""
     balance = await get_balance_command(
         normalize_number(contract_address), network=network
@@ -414,7 +458,8 @@ async def get_balance(contract_address, network):
 @cli.command()
 @click.argument("contract_address")
 @network_option
-async def get_nonce(contract_address, network):
+@enable_stack_trace
+async def get_nonce(ctx, contract_address, network):
     """Retrieve the nonce for a contract."""
     await get_nonce_command(normalize_number(contract_address), network)
 
