@@ -1,10 +1,13 @@
-"""nile common module."""
+"""Nile common module."""
+
 import json
 import os
 import re
+from pathlib import Path
 
 from starkware.crypto.signature.fast_pedersen_hash import pedersen_hash
 from starkware.starknet.core.os.class_hash import compute_class_hash
+from starkware.starknet.definitions.general_config import StarknetChainId
 from starkware.starknet.services.api.contract_class import ContractClass
 
 from nile.utils import normalize_number, str_to_felt
@@ -21,7 +24,7 @@ DEPLOYMENTS_FILENAME = "deployments.txt"
 DECLARATIONS_FILENAME = "declarations.txt"
 ACCOUNTS_FILENAME = "accounts.json"
 NODE_FILENAME = "node.json"
-RETRY_AFTER_SECONDS = 30
+RETRY_AFTER_SECONDS = 20
 TRANSACTION_VERSION = 1
 QUERY_VERSION_BASE = 2**128
 QUERY_VERSION = QUERY_VERSION_BASE + TRANSACTION_VERSION
@@ -31,11 +34,27 @@ UNIVERSAL_DEPLOYER_ADDRESS = (
     # subject to change
     "0x041a78e741e5af2fec34b695679bc6891742439f7afb8484ecd7766661ad02bf"
 )
+NETWORKS_CHAIN_ID = {
+    "localhost": StarknetChainId.TESTNET.value,
+    "mainnet": StarknetChainId.MAINNET.value,
+    "goerli": StarknetChainId.TESTNET.value,
+    "goerli2": StarknetChainId.TESTNET2.value,
+    "integration": StarknetChainId.TESTNET.value,
+}
 DEFAULT_GATEWAYS = {
     "localhost": "http://127.0.0.1:5050/",
     "goerli2": "https://alpha4-2.starknet.io",
     "integration": "https://external.integration.starknet.io",
 }
+_root = Path(__file__).parent.parent
+
+
+def contract_path(name):
+    """Return contract path."""
+    if name.startswith("tests/"):
+        return str(_root / name)
+    else:
+        return str(_root / "src" / name)
 
 
 def get_gateways():
@@ -154,6 +173,14 @@ def get_account_class_hash(contract="Account"):
     return get_class_hash(contract, overriding_path=overriding_path)
 
 
+def get_chain_id(network):
+    """Return the chain id given a network name, default to testnet."""
+    if network in NETWORKS_CHAIN_ID:
+        return NETWORKS_CHAIN_ID[network]
+    else:
+        return StarknetChainId.TESTNET.value
+
+
 def get_addresses_from_string(string):
     """Return a set of integers with identified addresses in a string."""
     return set(
