@@ -1,4 +1,5 @@
 """Command to run Nile scripts."""
+
 import logging
 from importlib.machinery import SourceFileLoader
 
@@ -8,7 +9,7 @@ from nile.nre import NileRuntimeEnvironment
 async def run(path, network):
     """Run nile scripts passing on the NRE object."""
     logger = logging.getLogger()
-    logger.disabled = True
+    logger.setLevel(logging.ERROR)
     script = SourceFileLoader("script", path).load_module()
     nre = NileRuntimeEnvironment(network)
     await script.run(nre)
